-------------------------------------------------------------------------------
-- Community Nicknames by Tammya-MoonGuard (2018)
--
-- All Rights Reserved.
-------------------------------------------------------------------------------
local _, Me = ...

-------------------------------------------------------------------------------
-- Expose to the outside world.
--
CommunityNicknames = Me

-------------------------------------------------------------------------------
-- A simple frame to listen for events. Right now we're only interested in the
--            ADDON_LOADED event to tell when we can hook the community frame.
Me.frame = CreateFrame( "Frame" )
Me.frame:RegisterEvent( "ADDON_LOADED" )
Me.frame:SetScript( "OnEvent", function( self, event, ... )
	if event == "ADDON_LOADED" then
		local addon = ...
		if addon == "Blizzard_Communities" then
			Me.HookCommunitiesFrame()
		end
	end
end)

-------------------------------------------------------------------------------
-- Hook for replacing names in the chat frame for when the user has added a 
--  community window to their chat frame. This works entirely different and is
--                 much simpler to edit than what goes in the community panel.
Me.hook_GetColoredName = GetColoredName
GetColoredName = function( ... )
	local event = ...
	if event == "CHAT_MSG_COMMUNITIES_CHANNEL" then
		local message = C_Club.GetInfoFromLastCommunityChatLine()
		if not message then return Me.hook_GetColoredName( ... ) end
		if message and message.author then
			local nickname = Me.ParseNickname( message.author.memberNote )
			if nickname then
				-- Ideally we wouldn't really block any other hooks to 
				--  GetColoredMessage, but there isn't really a safe way to
				--  continue execution while changing the player name (other
				--  things will still expect it to be a valid character name).
				return nickname
			end
		end
	end
	return Me.hook_GetColoredName( ... )
end

-------------------------------------------------------------------------------
-- Parses the [name] tag from a member note. This allows other addons to have
--  similarly formatted tags too. Anything before the [name] tag is ignored and
--                  it reads up until the next tag found (the opening bracket).
function Me.ParseNickname( member_note )
	if not member_note then return end
	-- Case insensitive match. Read until next bracket is found in the content.
	local nickname = member_note:match( "%[[Nn][Aa][Mm][Ee]%]([^%[]+)" )
	if nickname then
		-- Trim whitespace and cancel if the string is empty.
		nickname = nickname:match( "^%s*(.-)%s*$" )
		if nickname == "" then return nil end
		return nickname
	end
end

-------------------------------------------------------------------------------
-- Replaces the visible text in the first link found in a message. Might be 
--  better in the future if we actually verify that we're replacing the player
--  link. Might be some weird prefix that catches us, but as far as I know
--  there isn't anything that can be before the link aside from a timestamp.
-- This is only for the community frame, and not the chat frame replacements.
--
function Me.ChangeLinkText( message, visible_text )
	return message:gsub( "(%[|H.-|h).-|h", "%1" .. visible_text .. "|h", 1 )
end

-------------------------------------------------------------------------------
-- This took a bit to figure out. My first attempt was hooking 
--  GetBNPlayerCommunityLink and GetPlayerCommunityLink, but that taints the
--  community frame. My second attempt hooked CommunitiesFrame.Chat.AddMessage
--  and changed the message history in the scrolling message frame, but that
--  also taints everything. Now I'm hooking RefreshDisplay and iterating over
--  visible lines, and changing the text directly in the font string, so it's
--                    ONLY touching the UI stuff which should be fairly safe.
function Me.OnChatDisplayRefreshed( self )
	-- This little check is so any third party can disable us easily if they
	--  need to, for whatever reason.
	if not Me.refresh_display_hooked then return end
	
	
	for line_index, fontstring in pairs( self.visibleLines ) do
		local entry = self.historyBuffer:GetEntryAtIndex( 
		                                       line_index + self.scrollOffset )
		if entry then
			-- Club data is packed into the message. The full list is: 
			--  message, r, g, b, club_id, stream_id, message_id, member_id
			local message, _, _, _, club, _, _, member_id = self:UnpackageEntry( entry )
			
			-- One thing I hate about Lua is that there's no `continue`
			--  statement, so conditions can run pretty deep.
			if club and member_id then
				local author = C_Club.GetMemberInfo( club, member_id )
				-- As far as I know, all of this shit is nilable and needs to
				--  be checked.
				if author then
					local nickname = Me.ParseNickname( author.memberNote )
					if nickname then
						fontstring:SetText( 
						                Me.ChangeLinkText( message, nickname ))
					end
				end
			end
		end
	end
end

-------------------------------------------------------------------------------
-- Called after the community frame is loaded (Blizzard_Communities addon).
--
function Me.HookCommunitiesFrame()
	if not CommunitiesFrame then return end
	
	Me.refresh_display_hooked = true
	hooksecurefunc( CommunitiesFrame.Chat.MessageFrame, 
	                              "RefreshDisplay", Me.OnChatDisplayRefreshed )
end

Me.HookCommunitiesFrame()
